@@ -1,32 +1,32 @@
 """
 :class:`Property` objects can be defined on subclasses of
 :class:`~pydispatch.dispatch.Dispatcher` to create instance attributes that act
-as events when their values change::
+as events when their values change
 
-    from pydispatch import Dispatcher, Property
+.. doctest:: properties_module
 
-    class Foo(Dispatcher):
-        name = Property()
-        value = Property()
-    def __str__(self):
-        return self.__class__.__name__
+    >>> from pydispatch import Dispatcher, Property
 
-    class Listener(object):
-        def on_foo_name(self, instance, value, **kwargs):
-            print("{}'s name is {}".format(instance, value))
-        def on_foo_value(self, instance, value, **kwargs):
-            print('{} = {}'.format(instance, value))
+    >>> class Foo(Dispatcher):
+    ...     name = Property()
+    ...     value = Property()
+    ...     def __str__(self):
+    ...         return self.__class__.__name__
 
-    foo_obj = Foo()
-    listener_obj = Listener()
+    >>> class Listener(object):
+    ...     def on_foo_name(self, instance, value, **kwargs):
+    ...         print("{}'s name is {}".format(instance, value))
+    ...     def on_foo_value(self, instance, value, **kwargs):
+    ...         print('{} = {}'.format(instance, value))
 
-    foo_obj.bind(name=listener_obj.on_foo_name, value=listener_obj.on_foo_value)
+    >>> foo_obj = Foo()
+    >>> listener_obj = Listener()
+    >>> foo_obj.bind(name=listener_obj.on_foo_name, value=listener_obj.on_foo_value)
 
-    foo_obj.name = 'bar'
-    # Foo's name is bar
-
-    foo_obj.value = 42
-    # Foo = 42
+    >>> foo_obj.name = 'bar'
+    Foo's name is bar
+    >>> foo_obj.value = 42
+    Foo = 42
 
 Type checking is not enforced, so values can be any valid python type.
 Values are however checked for equality to avoid dispatching events for no
@@ -34,12 +34,12 @@ reason. If custom objects are used as values, they must be able to support
 equality checking. In most cases, this will be handled automatically.
 """
 
-import sys
 import weakref
 
 from pydispatch.utils import InformativeWVDict
 
-PY2 = sys.version_info < (3,)
+
+__all__ = ['Property', 'ListProperty', 'DictProperty']
 
 class Property(object):
     """Defined on the class level to create an observable attribute
@@ -62,11 +62,8 @@ class Property(object):
     @property
     def name(self):
         return self._name
-    @name.setter
-    def name(self, value):
-        if self._name != '':
-            return
-        self._name = value
+    def __set_name__(self, owner, name):
+        self._name = name
     def _add_instance(self, obj, default=None):
         if default is None:
             default = self.default
@@ -261,23 +258,10 @@ class ObservableList(list, Observable):
         old = self._get_copy_or_none()
         super(ObservableList, self).__delitem__(key)
         self._emit_change(old=old)
-    if PY2:
-        def __setslice__(self, *args):
-            old = self._get_copy_or_none()
-            super(ObservableList, self).__setslice__(*args)
-            self._emit_change(old=old)
-        def __delslice__(self, *args):
-            old = self._get_copy_or_none()
-            super(ObservableList, self).__delslice__(*args)
-            self._emit_change(old=old)
-    if hasattr(list, 'clear'):
-        def clear(self):
-            old = self._get_copy_or_none()
-            super(ObservableList, self).clear()
-            self._emit_change(old=old)
-    if not hasattr(list, 'copy'):
-        def copy(self):
-            return self[:]
+    def clear(self):
+        old = self._get_copy_or_none()
+        super(ObservableList, self).clear()
+        self._emit_change(old=old)
     def __iadd__(self, other):
         other = self._build_observable(other)
         self.extend(other)
